package de.hsbo.fbv.bmg.graphics.transforms.demos;

import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;
import java.util.HashMap;
import java.util.Map;

import de.hsbo.fbv.bmg.graphics.transforms.BilinearTransform;
import de.hsbo.fbv.bmg.graphics.transforms.TransformFactory;



/**
 * Testklasse, die die Berechnung der inversen Parameter eine bilinearen Transformation durchfhrt.
 * Mit diesem Beispiel lsst sich zeigen, dass die Umkehrabbildung einer bilinearen Transformation 
 * nicht fr alle Koordinaten durch eine einheitliche Transformationsmatrix angegeben werden kann, 
 * und daher als Lsungsansatz ausscheidet.  
 * @author zimmermann
 *
 */
public class InverseBilinearTransformTest {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// Ausgangstransformation festlegen
		BilinearTransform bt = new BilinearTransform();
		bt.translate(Math.random(), Math.random());
		bt.scale(Math.random(), Math.random());
		double alpha = Math.random()*Math.PI;
		bt.rotate(alpha, alpha);
		bt.converge(Math.random()*0.1, Math.random()*0.1);
		
		// Koordinaten zur Berechnung der Inversen festlegen
		Point2D[] oPts = {	new Point2D.Double(0.0, 0.0),
								new Point2D.Double(1.0, 0.0),
								new Point2D.Double(0.0, 1.0),
								new Point2D.Double(1.0, 1.0)};
		Map<Point2D,Point2D> ptmap = new HashMap<Point2D,Point2D>();
		
		// Identische Punkte in einer Map speichern
		for(int i=0; i<oPts.length; i++) 
			// ptmap.put(oPts[i],bt.transform(oPts[i]));
			ptmap.put(bt.transform(oPts[i]), oPts[i]);
		
		// Inverse Transformation berechnen
		BilinearTransform ibt = TransformFactory.create8Parameter(ptmap);
		
		// Ausgabe der beiden Transformationen
		System.out.println(bt);
		System.out.println();
		System.out.println(ibt);
		
		// Zufallspunkte berechnen und Differenzen feststellen
		for(int i=0; i<oPts.length; i++) {
			Point2D tp = ibt.transform(bt.transform(oPts[i]));
			// Point2D itp = bt.inverseTransform(oPts[i]);
			// Point2D tp = bt.transform(itp);
			System.out.println(oPts[i]+"\t "+tp+"\t "+oPts[i].distance(tp));
		}
		// Zufallspunkte berechnen und Differenzen feststellen
		for(int i=0; i<100; i++) {
			Point2D op = new Point2D.Double(Math.random(), Math.random());
			Point2D tp = ibt.transform(bt.transform(op));
			System.out.println(op+"\t "+tp+"\t "+op.distance(tp));
		}
		
	}

}
