package de.hsbo.fbv.bmg.graphics.transforms.demos;

import de.hsbo.fbv.bmg.graphics.transforms.Point2hD;
import de.hsbo.fbv.bmg.graphics.transforms.Shape2hD;

public class SpatialDataGenerator {
	
	static double[][] fr = null;
	static double[][] fi = null;
	static int di = 0;
	static int dj = 0;
	
	public static Shape2hD[] createGrid2hd(int n, double hf) {
		int k = 0;
		createFourierFactors(n, hf/3);
		Point2hD[][] pts = new Point2hD[n+1][n+1];
		for(int i=0; i<n+1; i++)
			for(int j=0; j<n+1; j++) {
				double z = linearTrend(i,j,n, 3*hf)+createFourier(i,j,n)+rand(hf/15);
				pts[i][j] = new SimplePoint2hD((i-n/2)*25, (j-n/2)*25, z);
			}
		Shape2hD[] shapes = new Shape2hD[2*(n+1)*(n+1)];
		for(int i=0; i<n; i++) {
			for(int j=0; j<n; j++) {
				shapes[k++] = new SimpleLine2hD(pts[i][j],pts[i][j+1]);
				shapes[k++] = new SimpleLine2hD(pts[i][j],pts[i+1][j]);
			}
		}
		for(int j=0; j<n; j++) {
			shapes[k++] = new SimpleLine2hD(pts[n][j],pts[n][j+1]);
			shapes[k++] = new SimpleLine2hD(pts[j][n],pts[j+1][n]);
		}
		return shapes;
	}

	public static Shape2hD[] createGrid2hd(int n, double dxy, double hf) {
		int k = 0;
		createFourierFactors(n, hf/3);
		Point2hD[][] pts = new Point2hD[n+1][n+1];
		for(int i=0; i<n+1; i++)
			for(int j=0; j<n+1; j++) {
				double z = linearTrend(i,j,n, 3*hf)+createFourier(i,j,n)+rand(hf/15);
				pts[i][j] = new SimplePoint2hD((i-n/2)*dxy, (j-n/2)*dxy, z);
			}
		Shape2hD[] shapes = new Shape2hD[2*(n+1)*(n+1)];
		for(int i=0; i<n; i++) {
			for(int j=0; j<n; j++) {
				shapes[k++] = new SimpleLine2hD(pts[i][j],pts[i][j+1]);
				shapes[k++] = new SimpleLine2hD(pts[i][j],pts[i+1][j]);
			}
		}
		for(int j=0; j<n; j++) {
			shapes[k++] = new SimpleLine2hD(pts[n][j],pts[n][j+1]);
			shapes[k++] = new SimpleLine2hD(pts[j][n],pts[j+1][n]);
		}
		return shapes;
	}

	private static void createFourierFactors(int n, double factor) {
		int m = n/10;
		fr = new double[m][m];
		fi = new double[m][m];
		for(int i=0; i<m; i++)
			for(int j=0; j<m; j++) {
				double phase = (Math.random()-Math.random())*2*Math.PI;
				fr[i][j] = factor/Math.pow(1.5, i+j)*Math.cos(phase);
				fi[i][j] = factor/Math.pow(1.5, i+j)*Math.sin(phase);
			}  
		di = n*(int)(Math.random()-Math.random());
		dj = n*(int)(Math.random()-Math.random());
	}
	
	private static double createFourier(int i, int j, int n) {
		int m = fr.length;
		double z = 0;
		double pi = 2*(i-di)*Math.PI/n;
		double pj = 2*(j-dj)*Math.PI/n;
		for(int im=0; im<m; im++)
			for(int jm=0; jm<m; jm++) {
				z = z + fr[im][jm]*Math.cos(im*pi)*Math.cos(jm*pj)+
						fi[im][jm]*Math.sin(im*pi)*Math.sin(jm*pj);
		}
		return z;
	}
	
	private static double linearTrend(int i, int j, int n, double factor) {
		return (i*1.0/n)*(j*1.0/n)*factor;
	}
	
	private static double rand(double x) {
		return x* (Math.random()-Math.random());
	}
}
