package de.hsbo.fbv.bmg.graphics.viewer.aufgaben;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.Stroke;

/**
 * Klasse, die geometrische Objekte vom Typ Shape zusammen mit ihrer graphischen Gestaltung speichert.
 * DecoratedShape hat die Rolle einer Factory-Klasse und wird in SimpleShapeViewer verwendet.
 * @author A.Zimmermann, HS Bochum, 2012
 *
 */
public class DecoratedShape {
	
	// Parameter
	static Stroke STROKE = new BasicStroke();
	static Paint PATTERN = Color.BLACK;
	
	// Attribute	
	Shape shape = null;
	Stroke stroke = null;
	Paint paint = null;
	
	private DecoratedShape(Shape shape, Stroke stroke, Paint paint) {
		this.shape = shape;
		this.stroke = stroke;
		this.paint = paint;
	}

	public static DecoratedShape createDrawShape(Shape shape) {
		return new DecoratedShape(shape, STROKE, PATTERN);
	}
	
	public static DecoratedShape createDrawShape(Shape shape, Stroke stroke) {
		return new DecoratedShape(shape, stroke, PATTERN);
	}
	
	public static DecoratedShape createDrawShape(Shape shape, Stroke stroke, Paint paint) {
		return new DecoratedShape(shape, stroke, paint);
	}
	
	public static DecoratedShape createDrawShape(Shape shape, Paint paint) {
		return new DecoratedShape(shape, STROKE, paint);
	}
	
	public static DecoratedShape createFillShape(Shape shape) {
		return new DecoratedShape(shape, null, PATTERN);
	}
	
	public static DecoratedShape createFillShape(Shape shape, Paint paint) {
		return new DecoratedShape(shape, null, paint);
	}
	
	public Shape getShape() {
		return shape;
	}

	public Stroke getStroke() {
		return stroke;
	}

	public Paint getPaint() {
		return paint;
	}
	
	public boolean isToBeDrawn() {
		if(stroke!=null) 
			return true;
		else
			return false;
	}
	
}
