package de.hsbo.fbv.bmg.graphics.viewer.aufgaben;

import java.awt.Dimension;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;

public class TransformFactory {
	
	public static double DeviceFrame = 10;
	
	/**
	 * Berechnet eine ebene Transformation vom Weltkoordinatensystem ins Gertekoordinatensystem.
	 * @param world 	Rechteckiger Ausschnitt des Weltkoordinatensystems 
	 * @param device	Gre des Darstellungsfensters (width, height)
	 * @return			Transformationsobjekt vom Typ AffineTransform
	 * @author 			A.Zimmermann, HS Bochum, 2011
	 */
	public static AffineTransform createSimpleWorldToDevice(Rectangle2D world, Dimension device) {
		try {
			// Berechnung der Transformationsparameter
			// 1. Schritt: Rcktranslation zum Mittelpunkt des Weltkoordinatenfensters
			double tx = world.getCenterX();
			double ty = world.getCenterY();
			// 2. Schritt: Skalierung, Spiegelung
			double mx = (device.getWidth()-2*DeviceFrame)/world.getWidth();
			double my = (device.getHeight()-2*DeviceFrame)/world.getHeight();
			double m = Math.min(my, mx);
			// 3. Schritt: Translation zum Mittelpunkt des Anzeigefensters
			double wx = device.getWidth()/2;
			double wy = device.getHeight()/2;
			// Anwendung der Parameter auf das Transformationsobjekt
			AffineTransform at = new AffineTransform();
			// 3. Schritt
			at.translate(wx, wy);
			// 2. Schritt
			at.scale(m, -m);
			// 1. Schritt
			at.translate(-tx, -ty);
			// Transformationsobjekt zurckgeben
			return at;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
	}
 
}
