package de.hsbo.fbv.bmg.graphics.viewer.commands;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import javax.swing.JFrame;
import javax.swing.JPanel;
import java.awt.Color;

/**
 * Adapterklasse fr die Schnittstelle Map.
 * @author zimmermann
 *
 */
public class MapAdapter extends JPanel implements Map {

	private static final long serialVersionUID = 1L;
	
	// Manuell eingefgte Attribute
	JPanel thisPanel = null;
	JFrame thisFrame = null;
	Command[] commands = new Command[20];
	Controller controller = null;
	Graphics graphics = null;
	Rectangle2D worldBounds = null;
	int state = 0;
	KeyEvent lastKeyEvent = null;
	KeyEvent actKeyEvent = null;
	MouseEvent lastMouseEvent = null;
	MouseEvent actMouseEvent = null;  //  @jve:decl-index=0:
	Point2D lastPoint = null;
	Point2D actPoint = null;
	// *****
	
	/**
	 * This is the default constructor
	 */
	public MapAdapter() {
		super();
		thisPanel = this;
		initialize();
	}
	
	/**
	 * Spezieller Konstruktor, der den Initialisierungsbefehl ausfhrt.
	 * @param initialize
	 */
	public MapAdapter(Command initialize) {
		super();
		thisPanel = this;
		initialize.execute();
		initialize();
	}

	/**
	 * This method initializes this
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 200);
		
		this.setLayout(null);
		this.setBackground(Color.white);
				
		this.addMouseListener(new java.awt.event.MouseAdapter() {   
			public void mouseReleased(java.awt.event.MouseEvent e) {
				runBefore(e);
				if(commands[Map.CmdMouseReleased]!=null)
					commands[Map.CmdMouseReleased].execute();
				runAfterMouseEvent();
			}   
			public void mousePressed(java.awt.event.MouseEvent e) {    
				runBefore(e);
				if(commands[Map.CmdMousePressed]!=null)
					commands[Map.CmdMousePressed].execute();
				runAfterMouseEvent();
			}   
			public void mouseExited(java.awt.event.MouseEvent e) {    
				runBefore(e);
				if(commands[Map.CmdMouseExited]!=null)
					commands[Map.CmdMouseExited].execute();
				runAfterMouseEvent();
			}   
			public void mouseEntered(java.awt.event.MouseEvent e) {    
				runBefore(e);
				if(commands[Map.CmdMouseEntered]!=null)
					commands[Map.CmdMouseEntered].execute();
				runAfterMouseEvent();
			}
			public void mouseClicked(java.awt.event.MouseEvent e) {
				runBefore(e);
				if(commands[Map.CmdMouseClicked]!=null)
					commands[Map.CmdMouseClicked].execute();
				runAfterMouseEvent();
			}
		});
		this.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {   
			public void mouseMoved(java.awt.event.MouseEvent e) {    
				runBefore(e);
				if(commands[Map.CmdMouseMoved]!=null)
					commands[Map.CmdMouseMoved].execute();
				runAfterMouseEvent();
			}
			public void mouseDragged(java.awt.event.MouseEvent e) {
				runBefore(e);
				if(commands[Map.CmdMouseDragged]!=null)
					commands[Map.CmdMouseDragged].execute();
				runAfterMouseEvent();
			}
		});
		this.addMouseWheelListener(new MyMouseWheelAdapter() {
			public void mouseWheelMoved(MouseWheelEvent e) {
				runBefore(e);
				if(commands[Map.CmdMouseWheelMoved]!=null)
					commands[Map.CmdMouseWheelMoved].execute();
				runAfterMouseEvent();
				repaint();
			}
		});
		this.addKeyListener(new java.awt.event.KeyAdapter() {   
			public void keyTyped(java.awt.event.KeyEvent e) { 
				actKeyEvent = e;
				if(commands[Map.CmdKeyTyped]!=null)
					commands[Map.CmdKeyTyped].execute();
				lastKeyEvent = actKeyEvent;
			}   
			public void keyReleased(java.awt.event.KeyEvent e) {    
				actKeyEvent = e;
				if(commands[Map.CmdKeyReleased]!=null)
					commands[Map.CmdKeyReleased].execute();
				lastKeyEvent = actKeyEvent;
			}
			public void keyPressed(java.awt.event.KeyEvent e) {
				actKeyEvent = e;
				if(commands[Map.CmdKeyPressed]!=null)
					commands[Map.CmdKeyPressed].execute();
				lastKeyEvent = actKeyEvent;
			}
		});
	}

	@Override
	public void initialize(Command command) {
		command.execute();
	}

	@Override
	public Controller getController() {
		return this.controller;
	}

	@Override
	public JFrame getFrame() {
		return this.thisFrame;
	}
	
	@Override
	public void setFrame(JFrame frame) {
		this.thisFrame = frame;
	}

	@Override
	public JPanel getPanel() {
		return thisPanel;
	}

	@Override
	public void setCommand(int i, Command command) {
		this.commands[i] = command;
	}

	@Override
	public void setController(Controller controller) {
		this.controller = controller;
	}
	
	@Override
	public AffineTransform getTransform() {
		return ((Graphics2D)this.getGraphics()).getTransform();
	}

	@Override
	public Rectangle2D getWorldBounds() {
		return this.worldBounds;
	}

	@Override
	public void setTransform(AffineTransform at) {
		((Graphics2D)this.getGraphics()).setTransform(at);
	}

	@Override
	public void setWorldBounds(Rectangle2D r) {
		this.worldBounds = r;
	}
	
	@Override
	public void executeCommand(int cmd) {
		this.commands[cmd].execute();
	}

	@Override
	public int getState() {
		return this.state;
	}

	@Override
	public void setState(int state) {
		this.state = state;
	}

	@Override
	public Graphics getGraphics() {
		return this.graphics;
	}
	
	@Override
	public KeyEvent getLastKeyEvent() {
		return lastKeyEvent;
	}
	
	@Override
	public KeyEvent getActKeyEvent() {
		return actKeyEvent;
	}

	@Override
	public MouseEvent getLastMouseEvent() {
		return lastMouseEvent;
	}

	@Override
	public MouseEvent getActMouseEvent() {
		return actMouseEvent;
	}

	@Override
	public Point2D getLastPoint() {
		return lastPoint;
	}

	@Override
	public Point2D getActPoint() {
		return actPoint;
	}
	
	@Override
	public AffineTransform getTransform(Rectangle2D world) {
		Dimension dimension = this.getSize();
		// 1. Translation
		double tx1 = world.getCenterX();
		double ty1 = world.getCenterY();
		// 2. Skalieren und Spiegeln
		double width = world.getWidth();
		double height = world.getHeight();
		double s2 = Math.min(dimension.width/width, dimension.height/height);
		// 3. Rcktranslation
		double tx3 = dimension.width/2;
		double ty3 = dimension.height/2;
		AffineTransform at = new AffineTransform();
		at.translate(tx3, ty3);
		at.scale(s2, -s2);
		at.translate(-tx1, -ty1);
		// Ergebnis
		return at;
	}

	/*
	public void paint(Graphics g) {
		this.graphics = g;
		// g.clearRect(0, 0, this.getWidth(), this.getHeight());
		if(commands[Map.CmdPaint]!=null) 
			commands[Map.CmdPaint].execute();
	}
	*/
	
	public void paintComponent(Graphics g) {
		this.graphics = g;
		g.clearRect(0, 0, this.getWidth(), this.getHeight());
		if(commands[Map.CmdPaint]!=null) 
			commands[Map.CmdPaint].execute();
	}
	
	private void runBefore(MouseEvent e) {
		try {
			this.actMouseEvent = e;
			AffineTransform at = ((Graphics2D)this.getGraphics()).getTransform();
			this.actPoint = at.inverseTransform(new Point2D.Double(e.getX(), e.getY()), null);
		} catch (NoninvertibleTransformException e1) {
			// TODO Auto-generated catch block
			// e1.printStackTrace();
			this.actPoint = null;
		}
	}
	
	private void runAfterMouseEvent() {
		this.lastMouseEvent = this.actMouseEvent;
		this.lastPoint = this.actPoint;
	}
	
}
