package de.hsbo.fbv.bmg.graphics.viewer.simple;

import java.awt.Color;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Path2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.List;

/**
 * ShapeLayer - Adapterklasse des Interface Layer zur Aufnahme einfacher Shape-Objekte.<br>
 * @author Zimmermann, HS Bochum 2011
 *
 */
public class ShapeLayer<ElemT extends Shape> implements Layer {
	
	// Attribute
	boolean visible = true;
	Color fillColor = null;
	Color drawColor = null;
	Stroke stroke = null;
	List<ElemT> list = new ArrayList<ElemT>();
	

	@Override
	public Rectangle2D getBounds2D() {
		Path2D path = new Path2D.Double();
		for(int i=0; i<list.size(); i++) {
			path.append(list.get(i).getBounds2D(), false);
		}
		return path.getBounds2D();
	}

	@Override
	public StyledShapes[] getStyledShapes() {
		if(!isVisible()) {
			return null;
		}
		else if(this.fillColor==null && this.drawColor==null)
			return null;
		else {
			StyledShapes styledShapes = null;
			ArrayList<StyledShapes> listShapes = new ArrayList<StyledShapes>();
			Shape[] shapes = new Shape[list.size()];
			list.toArray(shapes);
			if(this.fillColor!=null) {
				styledShapes = new StyledShapes(shapes);
				styledShapes.setFill();
				styledShapes.setColor(this.fillColor);
				listShapes.add(styledShapes);
			}
			if(this.drawColor!=null) {
				styledShapes = new StyledShapes(shapes);
				styledShapes.setDraw();
				styledShapes.setColor(this.drawColor);
				styledShapes.setStroke(this.stroke);
				listShapes.add(styledShapes);
			}
			StyledShapes[] result = new StyledShapes[listShapes.size()];
			return listShapes.toArray(result);
		}
	}

	@Override
	public boolean isVisible() {
		return visible;
	}

	public Color getFillColor() {
		return fillColor;
	}

	public void setFillColor(Color fill) {
		this.fillColor = fill;
	}

	public Color getDrawColor() {
		return drawColor;
	}

	public void setDrawColor(Color draw) {
		this.drawColor = draw;
	}

	public Stroke getStroke() {
		return stroke;
	}

	public void setStroke(Stroke stroke) {
		this.stroke = stroke;
	}

	public void setVisible(boolean visible) {
		this.visible = visible;
	}

	public int size() {
		return list.size();
	}

	public boolean isEmpty() {
		return list.isEmpty();
	}

	public boolean contains(Object o) {
		return list.contains(o);
	}

	public boolean add(ElemT e) {
		return list.add(e);
	}

	public boolean remove(Object o) {
		return list.remove(o);
	}

	public ElemT get(int index) {
		return list.get(index);
	}

	public ElemT set(int index, ElemT element) {
		return list.set(index, element);
	}

	public void add(int index, ElemT element) {
		list.add(index, element);
	}

	public ElemT remove(int index) {
		return list.remove(index);
	}

	public int indexOf(Object o) {
		return list.indexOf(o);
	}

}
